import 'dart:io';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hive/hive.dart';
import 'package:hydrated_bloc/hydrated_bloc.dart';
import 'package:path_provider/path_provider.dart';

import 'package:scrobbles/config/theme.dart';
import 'package:scrobbles/cubit/bottom_nav_cubit.dart';
import 'package:scrobbles/cubit/data_counts_by_day_cubit.dart';
import 'package:scrobbles/cubit/data_counts_by_hour_cubit.dart';
import 'package:scrobbles/cubit/data_discoveries_cubit.dart';
import 'package:scrobbles/cubit/data_heatmap_cubit.dart';
import 'package:scrobbles/cubit/data_new_artists_cubit.dart';
import 'package:scrobbles/cubit/data_new_tracks_cubit.dart';
import 'package:scrobbles/cubit/data_statistics_global_cubit.dart';
import 'package:scrobbles/cubit/data_statistics_recent_cubit.dart';
import 'package:scrobbles/cubit/data_timeline_cubit.dart';
import 'package:scrobbles/cubit/data_top_artists_cubit.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/ui/skeleton.dart';

void main() async {
  /// Initialize packages
  WidgetsFlutterBinding.ensureInitialized();
  await EasyLocalization.ensureInitialized();
  final Directory tmpDir = await getTemporaryDirectory();
  Hive.init(tmpDir.toString());
  HydratedBloc.storage = await HydratedStorage.build(
    storageDirectory: tmpDir,
  );

  runApp(
    EasyLocalization(
      path: 'assets/translations',
      supportedLocales: const <Locale>[
        Locale('en'),
        Locale('fr'),
      ],
      fallbackLocale: const Locale('en'),
      useFallbackTranslations: true,
      child: const MyApp(),
    ),
  );
}

class MyApp extends StatelessWidget {
  const MyApp({super.key});

  @override
  Widget build(BuildContext context) {
    return MultiBlocProvider(
      providers: [
        BlocProvider<SettingsCubit>(create: (context) => SettingsCubit()),
        BlocProvider<BottomNavCubit>(create: (context) => BottomNavCubit()),
        BlocProvider<DataCountsByDayCubit>(create: (context) => DataCountsByDayCubit()),
        BlocProvider<DataCountsByHourCubit>(create: (context) => DataCountsByHourCubit()),
        BlocProvider<DataDiscoveriesCubit>(create: (context) => DataDiscoveriesCubit()),
        BlocProvider<DataHeatmapCubit>(create: (context) => DataHeatmapCubit()),
        BlocProvider<DataNewArtistsCubit>(create: (context) => DataNewArtistsCubit()),
        BlocProvider<DataNewTracksCubit>(create: (context) => DataNewTracksCubit()),
        BlocProvider<DataStatisticsGlobalCubit>(
            create: (context) => DataStatisticsGlobalCubit()),
        BlocProvider<DataStatisticsRecentCubit>(
            create: (context) => DataStatisticsRecentCubit()),
        BlocProvider<DataTimelineCubit>(create: (context) => DataTimelineCubit()),
        BlocProvider<DataTopArtistsCubit>(create: (context) => DataTopArtistsCubit()),
      ],
      child: MaterialApp(
        title: 'Scrobbles',
        theme: appTheme,
        home: const SkeletonScreen(),

        // Localization stuff
        localizationsDelegates: context.localizationDelegates,
        supportedLocales: context.supportedLocales,
        locale: context.locale,
        debugShowCheckedModeBanner: false,
      ),
    );
  }
}
